@@ -84,8 +84,7 @@ sub run {
         $self->logger->debug( " -> Send challenge: " . $data->{challenge} );
 
         # Serialize data
-        $data = to_json(
-            {
+        $data = to_json( {
                 challenge      => $data->{challenge},
                 appId          => $data->{appId},
                 registeredKeys => \@rk
