@@ -1,9 +1,13 @@
 package Lemonldap::NG::Portal::Auth::SAML;
 
 use strict;
+use MIME::Base64 qw/encode_base64/;
 use Mouse;
+use Lemonldap::NG::Portal::Lib::SAML;
+use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_CONFIRM
+  PE_IDPCHOICE
   PE_LOGOUT_OK
   PE_REDIRECT
   PE_OK
@@ -20,15 +24,19 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SENDRESPONSE
 );
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.3';
 
-extends 'Lemonldap::NG::Portal::Auth::Base', 'Lemonldap::NG::Portal::Lib::SAML';
+extends 'Lemonldap::NG::Portal::Main::Auth', 'Lemonldap::NG::Portal::Lib::SAML';
 
 # INTERFACE
 
-has sloAssConsumerRe => ( is => 'rw' );
+has ssoAssConsumerRe => ( is => 'rw' );
 has sloRe            => ( is => 'rw' );
 has artRe            => ( is => 'rw' );
+has catch            => ( is => 'rw' );
+use constant sessionKind => 'SAML';
+use constant lsDump      => '_lassoSessionDump';
+use constant liDump      => '_lassoIdentityDump';
 
 sub forAuthUser { 'handleAuthRequests' }
 
@@ -43,7 +51,7 @@ sub init {
         "samlSPSSODescriptorAssertionConsumerServiceHTTPPost");
     my $saml_acs_get_url = $self->getMetaDataURL(
         "samlSPSSODescriptorAssertionConsumerServiceHTTPRedirect");
-    $self->sloAssConsumerRe(
+    $self->ssoAssConsumerRe(
         qr/^($saml_acs_art_url|$saml_acs_post_url|$saml_acs_get_url)(?:\?.*)?$/i
     );
     my $saml_slo_soap_url =
@@ -67,6 +75,9 @@ qr/^($saml_slo_soap_url|$saml_slo_soap_url_ret|$saml_slo_get_url|$saml_slo_get_u
     my $saml_ars_url = $self->getMetaDataURL(
         "samlSPSSODescriptorArtifactResolutionServiceArtifact");
     $self->artRe(qr/^($saml_ars_url)(?:\?.*)?$/i);
+    $self->catch(
+qr/^($saml_acs_art_url|$saml_acs_post_url|$saml_acs_get_url$saml_slo_soap_url|$saml_slo_soap_url_ret|$saml_slo_get_url|$saml_slo_get_url_ret|$saml_slo_post_url|$saml_slo_post_url_ret)(?:\?.*)?$/i
+    );
 
     # Load SAML service and SAML IdP list
     return ( $self->SUPER::init and $self->loadIDPs );
@@ -84,7 +95,7 @@ sub extractFormInfo {
     my $content_type   = $req->content_type;
 
     # 1.1 SSO assertion consumer
-    if ( $url =~ $self->sloAssConsumerRe ) {
+    if ( $url =~ $self->ssoAssConsumerRe ) {
         $self->logger->debug(
             "URL $url detected as an SSO assertion consumer URL");
 
@@ -187,6 +198,7 @@ sub extractFormInfo {
                     delete $req->{urldc};
                     $req->mustRedirect(1);
                     $req->steps( [] );
+                    $req->continue(1);
                     return PE_OK;
                 }
             }
@@ -228,7 +240,8 @@ sub extractFormInfo {
               ->{samlIDPMetaDataOptionsRelayStateURL};
 
             #  Extract RelayState information
-            if ( $self->extractRelayState( $relaystate, $relayStateURL ) ) {
+            if ( $self->extractRelayState( $req, $relaystate, $relayStateURL ) )
+            {
                 $self->logger->debug("RelayState $relaystate extracted");
             }
 
@@ -281,44 +294,85 @@ sub extractFormInfo {
             }
 
             # Get NameID
-            my $nameid = $login->nameIdentifier;
+            my $nameid         = $login->nameIdentifier;
+            my $nameid_content = $nameid->content;
 
-            # Set user
-            my $user = $nameid->content;
-
-            unless ($user) {
+            unless ($nameid_content) {
                 $self->userLogger->error("No NameID value found");
                 return PE_SAML_SSO_ERROR;
             }
 
-            $self->logger->debug("Found NameID: $user");
+            $self->logger->debug("Found NameID content $nameid_content");
+
+            # Set user
+            my $user = $nameid_content;
+            my $userAttribute =
+              $self->conf->{samlIDPMetaDataOptions}->{$idpConfKey}
+              ->{samlIDPMetaDataOptionsUserAttribute};
+
+            if ($userAttribute) {
+                $self->logger->debug(
+                    "Try to set user value from SAML attribute $userAttribute");
+
+                my $force_utf8 =
+                  $self->conf->{samlIDPMetaDataOptions}->{$idpConfKey}
+                  ->{samlIDPMetaDataOptionsForceUTF8};
+
+                my $attr_statement = $assertion->AttributeStatement();
+                if ($attr_statement) {
+
+                    # Get attributes
+                    my @attributes = $attr_statement->Attribute();
+                    #
+                    # Try to get value
+                    my $value =
+                      $self->getAttributeValue( $userAttribute, undef, undef,
+                        \@attributes, $force_utf8 );
+
+                    # Store value as user
+                    if ( defined $value ) {
+                        $user = $value;
+                        $self->logger->debug(
+                            "Found value $value for attribute $userAttribute");
+                    }
+                }
+                else {
+                    $self->logger->debug(
+                        "No attributes found in SAML assertion");
+                }
+            }
+
             $req->user($user);
+            $self->logger->debug("Set user value to $user");
 
             # Store Lasso objects
-            $req->datas->{_lassoLogin}   = $login;
-            $req->datas->{_idp}          = $idp;
-            $req->datas->{_idpConfKey}   = $idpConfKey;
-            $req->datas->{_nameID}       = $nameid;
-            $req->datas->{_sessionIndex} = $session_index;
+            $req->data->{_lassoLogin}   = $login;
+            $req->data->{_idp}          = $idp;
+            $req->data->{_idpConfKey}   = $idpConfKey;
+            $req->data->{_nameID}       = $nameid;
+            $req->data->{_sessionIndex} = $session_index;
 
             # Store Token
             my $saml_token = $assertion->export_to_xml;
 
             $self->logger->debug("SAML Token: $saml_token");
 
-            $req->datas->{_samlToken} = $saml_token;
+            $req->data->{_samlToken} = $saml_token;
 
             # Restore initial SAML request in case of proxying
             if ($assertion_responded) {
-                my $moduleOptions = $self->conf->{samlStorageOptions} || {};
-                $moduleOptions->{backend} = $self->conf->{samlStorage};
-                my $module = "Lemonldap::NG::Common::Apache::Session";
 
                 my $saml_sessions =
-                  $module->searchOn( $moduleOptions, "ProxyID",
-                    $assertion_responded );
+                  Lemonldap::NG::Common::Apache::Session->searchOn(
+                    $self->amOpts, "ProxyID", $assertion_responded );
 
-                if ( my @saml_sessions_keys = keys %$saml_sessions ) {
+                if (
+                    my @saml_sessions_keys = grep {
+                        $saml_sessions->{$_}->{_session_kind} eq
+                          $self->sessionKind
+                    } keys %$saml_sessions
+                  )
+                {
 
                     # Warning if more than one session found
                     if ( $#saml_sessions_keys > 0 ) {
@@ -337,15 +391,25 @@ sub extractFormInfo {
 
                     my $samlSessionInfo = $self->getSamlSession($saml_session);
 
-                    $req->datas->{_proxiedRequest} =
+                    $req->data->{_proxiedRequest} =
                       $samlSessionInfo->data->{Request};
-                    $req->datas->{_proxiedMethod} =
+                    $req->data->{_proxiedMethod} =
                       $samlSessionInfo->data->{Method};
-                    $req->datas->{_proxiedRelayState} =
+                    $req->data->{_proxiedRelayState} =
                       $samlSessionInfo->data->{RelayState};
-                    $req->datas->{_proxiedArtifact} =
+                    $req->data->{_proxiedArtifact} =
                       $samlSessionInfo->data->{Artifact};
 
+               # Save values in hidden fields in case of other user interactions
+                    $self->p->setHiddenFormValue( $req, 'SAMLRequest',
+                        $self->{_proxiedRequest} );
+                    $self->p->setHiddenFormValue( $req, 'Method',
+                        $self->{_proxiedMethod} );
+                    $self->p->setHiddenFormValue( $req, 'RelayState',
+                        $self->{_proxiedRelayState} );
+                    $self->p->setHiddenFormValue( $req, 'SAMLart',
+                        $self->{_proxiedArtifact} );
+
                     # Delete session
                     $samlSessionInfo->remove();
                 }
@@ -456,7 +520,7 @@ sub extractFormInfo {
             }
 
             # If URL in RelayState, different from portal, redirect user
-            if ( $self->extractRelayState($relaystate) ) {
+            if ( $self->extractRelayState( $req, $relaystate ) ) {
                 $self->logger->debug("RelayState $relaystate extracted");
                 $self->logger->debug(
                     "URL " . $req->urldc . " found in RelayState" );
@@ -543,14 +607,17 @@ sub extractFormInfo {
             $self->logger->debug("Logout request NameID content: $user");
 
             # Get SAML sessions with the same NameID
-            my $moduleOptions = $self->conf->{samlStorageOptions} || {};
-            $moduleOptions->{backend} = $self->conf->{samlStorage};
-            my $module = "Lemonldap::NG::Common::Apache::Session";
 
             my $local_sessions =
-              $module->searchOn( $moduleOptions, "_nameID", $name_id->dump );
+              Lemonldap::NG::Common::Apache::Session->searchOn( $self->amOpts,
+                "_nameID", $name_id->dump );
 
-            if ( my @local_sessions_keys = keys %$local_sessions ) {
+            if (
+                my @local_sessions_keys = grep {
+                    $local_sessions->{$_}->{_session_kind} eq $self->sessionKind
+                } keys %$local_sessions
+              )
+            {
 
                 # At least one session was found
                 foreach (@local_sessions_keys) {
@@ -590,13 +657,13 @@ sub extractFormInfo {
                   # Get Lasso::Session dump
                   # This value is erased if a next session match the SLO request
                         if (   $ssoSession
-                            && $ssoSession->data->{_lassoSessionDump} )
+                            && $ssoSession->data->{ $self->lsDump } )
                         {
                             $self->logger->debug(
 "Get Lasso::Session dump from session $real_session"
                             );
                             $session_dump =
-                              $ssoSession->data->{_lassoSessionDump};
+                              $ssoSession->data->{ $self->lsDump };
                         }
 
                 # Real session will be deleted after (see $req->steps... before)
@@ -671,7 +738,6 @@ sub extractFormInfo {
 
             # Send response depending on request method
             # HTTP-REDIRECT
-            no strict 'subs';
             if ( $method == Lasso::Constants::HTTP_METHOD_REDIRECT ) {
 
                 # Redirect user to response URL
@@ -709,8 +775,7 @@ sub extractFormInfo {
 
                 $self->logger->debug("SOAP response $slo_body");
 
-                $req->response(
-                    [
+                $req->response( [
                         200,
                         [
                             'Content-Type'   => 'application/xml',
@@ -719,15 +784,14 @@ sub extractFormInfo {
                         [$slo_body]
                     ]
                 );
-                $req->steps(
-                    [
+                $req->steps( [
                         @{ $self->p->beforeLogout },
                         sub {
                             my ($req) = @_;
                             my $res = $self->p->deleteSession($req);
                             return (
                                 $res eq PE_LOGOUT_OK ? PE_SENDRESPONSE : $res );
-                          }
+                        }
                     ]
                 );
 
@@ -779,8 +843,7 @@ sub extractFormInfo {
             return PE_SAML_ART_ERROR;
         }
 
-        $req->response(
-            [
+        $req->response( [
                 200,
                 [
                     'Content-Type'   => 'application/xml',
@@ -798,12 +861,75 @@ sub extractFormInfo {
     # Search a selected IdP
     my ( $idp, $idp_cookie ) = $self->getIDP($req);
 
-    # Get confirmation flag
-    my $confirm_flag = $req->param("confirm") // 0;
+    # Use Common Domain Cookie
+    if (   !$idp
+        and $self->conf->{samlCommonDomainCookieActivation}
+        and $self->conf->{samlCommonDomainCookieReader} )
+    {
+        $self->logger->debug(
+            "Will try to use Common Domain Cookie for IDP resolution");
 
-    # If confirmation is -1 from resolved IDP screen,
-    # or IDP was not resolve, let the user choose its IDP
-    if ( $confirm_flag == -1 or !$idp ) {
+        # Add current URL to CDC Reader URL
+        my $return_url = encode_base64( $self->p->fullUrl($req), '' );
+
+        my $cdc_reader_url = $self->conf->{samlCommonDomainCookieReader};
+
+        $cdc_reader_url .= ( $cdc_reader_url =~ /\?/ ? '&' : '?' )
+          . build_urlencoded( url => $return_url );
+
+        $self->logger->debug("Redirect user to $cdc_reader_url");
+
+        $req->urldc($cdc_reader_url);
+
+        $req->steps( [] );
+        return PE_REDIRECT;
+    }
+
+    # Use SAML Discovery Protocol
+    if (   !$idp
+        and $self->conf->{samlDiscoveryProtocolActivation}
+        and defined $self->conf->{samlDiscoveryProtocolURL} )
+    {
+        $self->logger->debug(
+            "Will try to use SAML Discovery Protocol for IDP resolution");
+
+        if ( $req->urldc ) {
+            $req->pdata->{_url} = encode_base64( $req->urldc, '' );
+        }
+        my $disco_url = $self->conf->{samlDiscoveryProtocolURL};
+        my $portal    = $self->conf->{portal};
+        $disco_url .= ( $disco_url =~ /\?/ ? '&' : '?' )
+          . build_urlencoded(
+            entityID      => $self->getMetaDataURL( 'samlEntityID', 0, 1 ),
+            return        => $portal,
+            returnIDParam => 'idp'
+          );
+
+        if ( defined $self->conf->{samlDiscoveryProtocolPolicy} ) {
+            my $dppolicy = $self->conf->{samlDiscoveryProtocolPolicy};
+            $disco_url .= "&" . build_urlencoded( policy => $dppolicy );
+        }
+
+        if ( defined $self->conf->{samlDiscoveryProtocolIsPassive} ) {
+            my $dpispassive = $self->conf->{samlDiscoveryProtocolIsPassive};
+            $disco_url .= "&"
+              . build_urlencoded(
+                isPassive => $dpispassive
+                ? "true"
+                : "false"
+              );
+        }
+
+        $self->logger->debug("Redirect user to $disco_url");
+
+        $req->urldc($disco_url);
+
+        $req->steps( [] );
+        return PE_REDIRECT;
+    }
+
+    # If IDP was not resolved, let the user choose its IDP
+    unless ($idp) {
         $self->logger->debug("Redirecting user to IDP list");
 
         # Control url parameter
@@ -811,16 +937,43 @@ sub extractFormInfo {
         return $urlcheck unless ( $urlcheck == PE_OK );
 
         # IDP list
-        my @list = ();
+        my @list       = ();
+        my $portalPath = $self->conf->{portal};
+
         foreach ( keys %{ $self->idpList } ) {
+            my $idpName = $self->{idpList}->{$_}->{name};
+            $idpName = $self->{idpList}->{$_}->{displayName}
+              if $self->{idpList}->{$_}->{displayName};
+            my $icon    = $self->{idpList}->{$_}->{icon};
+            my $order   = $self->{idpList}->{$_}->{order} // 0;
+            my $img_src = '';
+
+            if ($icon) {
+                $img_src =
+                  ( $icon =~ m#^https?://# )
+                  ? $icon
+                  : $portalPath . $self->p->staticPrefix . "/common/" . $icon;
+            }
+            $self->logger->debug( "IDP "
+                  . $self->{idpList}->{$_}->{name}
+                  . " -> DisplayName : $idpName with Icon : $img_src at order : $order"
+            );
             push @list,
               {
-                val  => $_,
-                name => $self->idpList->{$_}->{name}
+                val   => $_,
+                name  => $idpName,
+                icon  => $img_src,
+                order => $order,
               };
         }
-        $req->datas->{list}            = \@list;
-        $req->datas->{confirmRemember} = 1;
+        @list =
+          sort {
+                 $a->{order} <=> $b->{order}
+              or $a->{name} cmp $b->{name}
+              or $a->{val} cmp $b->{val}
+          } @list;
+        $req->data->{list}            = \@list;
+        $req->data->{confirmRemember} = 1;
 
         # Delete existing IDP resolution cookie
         $req->addCookie(
@@ -829,45 +982,16 @@ sub extractFormInfo {
                 value   => 0,
                 domain  => $self->conf->{domain},
                 secure  => 0,
-                expires => '-1d',
+                expires => 'Wed, 21 Oct 2015 00:00:00 GMT',
             )
         );
 
         #TODO: check this
-        $req->datas->{login} = 1;
-        return PE_CONFIRM;
+        $req->data->{login} = 1;
+        return PE_IDPCHOICE;
     }
 
-    # If IDP is found but not confirmed, let the user confirm it
-    elsif ( $confirm_flag != 1 ) {
-        $self->logger->debug("IDP $idp selected, need user confirmation");
-
-        # Control url parameter
-        my $urlcheck = $self->p->controlUrl($req);
-        return $urlcheck unless ( $urlcheck == PE_OK );
-
-        # Choosen IDP
-        my $html =
-'<h3 trspan="redirectionToIdp">Redirection to your Identity Provider</h3><h4>'
-          . $self->idpList->{$idp}->{name}
-          . "</h4>\n"
-          . "<p><i>"
-          . $idp
-          . "</i></p>\n"
-          . (
-            $req->param("url")
-            ? "<input type=\"hidden\" name=\"url\" value=\""
-              . $req->param("url") . "\" />"
-            : ''
-          ) . "<input type=\"hidden\" name=\"idp\" value=\"$idp\" />\n";
-
-        $req->info($html);
-
-        $req->datas->{login} = 1;
-        return PE_CONFIRM;
-    }
-
-    # Here confirmation is OK (confirm_flag == 1), store choosen IDP in cookie
+    # Store choosen IDP in cookie
     unless ( $idp_cookie and $idp eq $idp_cookie ) {
         $self->logger->debug("Build cookie to remember $idp as IDP choice");
 
@@ -878,13 +1002,14 @@ sub extractFormInfo {
         # User can choose temporary (0) or persistent cookie (1)
         my $cookie_type = $req->param("cookie_type") || "0";
 
+        # Cookie available 1 year
         $req->addCookie(
             $self->p->cookie(
-                name    => $self->conf->{samlIdPResolveCookie},
-                value   => $idp,
-                domain  => $self->conf->{domain},
-                secure  => $self->conf->{securedCookie},
-                expires => $cookie_type ? "+365d" : "",
+                name   => $self->conf->{samlIdPResolveCookie},
+                value  => $idp,
+                domain => $self->conf->{domain},
+                secure => $self->conf->{securedCookie},
+                ( $cookie_type ? ( max_age => 31557600 ) : () ),
             )
         );
     }
@@ -925,7 +1050,6 @@ sub extractFormInfo {
 
     # If no method defined, get first HTTP method
     unless ( defined $method ) {
-        no strict 'subs';
         my $protocolType = Lasso::Constants::MD_PROTOCOL_TYPE_SINGLE_SIGN_ON;
         $method =
           $self->getFirstHttpMethod( $self->lassoServer, $idp, $protocolType );
@@ -977,16 +1101,16 @@ sub extractFormInfo {
     }
 
     # Keep initial SAML request data in memory in case of proxing
-    if ( $req->datas->{_proxiedSamlRequest} ) {
+    if ( $req->data->{_proxiedSamlRequest} ) {
 
         my $infos;
 
         $infos->{type}       = 'proxy';
         $infos->{_utime}     = time;
-        $infos->{Request}    = $req->datas->{_proxiedRequest};
-        $infos->{Method}     = $req->datas->{_proxiedMethod};
-        $infos->{RelayState} = $req->datas->{_proxiedRelayState};
-        $infos->{Artifact}   = $req->datas->{_proxiedArtifact};
+        $infos->{Request}    = $req->data->{_proxiedRequest};
+        $infos->{Method}     = $req->data->{_proxiedMethod};
+        $infos->{RelayState} = $req->data->{_proxiedRelayState};
+        $infos->{Artifact}   = $req->data->{_proxiedArtifact};
         $infos->{ProxyID}    = $samlID;
 
         my $samlSessionInfo = $self->getSamlSession( undef, $infos );
@@ -1050,9 +1174,9 @@ sub authenticate {
 
 sub setAuthSessionInfo {
     my ( $self, $req ) = @_;
-    my $login      = $req->datas->{_lassoLogin};
-    my $idp        = $req->datas->{_idp};
-    my $idpConfKey = $req->datas->{_idpConfKey};
+    my $login      = $req->data->{_lassoLogin};
+    my $idp        = $req->data->{_idp};
+    my $idpConfKey = $req->data->{_idpConfKey};
 
     # Get SAML assertion
     my $assertion = $self->getAssertion($login);
@@ -1143,21 +1267,21 @@ sub setAuthSessionInfo {
     my $identity = $login->get_identity;
 
     # Dump Lasso objects in session
-    $req->{sessionInfo}->{_lassoSessionDump}  = $session->dump()  if $session;
-    $req->{sessionInfo}->{_lassoIdentityDump} = $identity->dump() if $identity;
+    $req->{sessionInfo}->{ $self->lsDump } = $session->dump()  if $session;
+    $req->{sessionInfo}->{ $self->liDump } = $identity->dump() if $identity;
 
     # Keep SAML Token in session
     my $store_samlToken = $self->conf->{samlIDPMetaDataOptions}->{$idpConfKey}
       ->{samlIDPMetaDataOptionsStoreSAMLToken};
     if ($store_samlToken) {
         $self->logger->debug("Store SAML Token in session");
-        $req->{sessionInfo}->{_samlToken} = $req->datas->{_samlToken};
+        $req->{sessionInfo}->{_samlToken} = $req->data->{_samlToken};
     }
     else {
         $self->logger->debug("SAML Token will not be stored in session");
     }
 
-    $req->datas->{_lassoLogin} = $login;
+    $req->data->{_lassoLogin} = $login;
     push @{ $req->steps }, sub { $self->authFinish(@_) };
 
     PE_OK;
@@ -1173,8 +1297,8 @@ sub authFinish {
     my $utime = $req->{sessionInfo}->{_utime};
 
     # Get saved Lasso objects
-    my $nameid        = $req->datas->{_nameID};
-    my $session_index = $req->datas->{_sessionIndex};
+    my $nameid        = $req->data->{_nameID};
+    my $session_index = $req->data->{_sessionIndex};
 
     $self->logger->debug( "Store NameID "
           . $nameid->dump
@@ -1211,7 +1335,7 @@ sub authLogout {
     $self->deleteSAMLSecondarySessions($session_id);
 
     # Recover Lasso::Session dump
-    my $session_dump = $req->{sessionInfo}->{_lassoSessionDump};
+    my $session_dump = $req->{sessionInfo}->{ $self->lsDump };
 
     unless ($session_dump) {
         $self->userLogger->error("Could not get session dump from session");
@@ -1224,7 +1348,6 @@ sub authLogout {
     $method = $self->getHttpMethod($method);
 
     # If no method defined, get first HTTP method
-    no strict 'subs';
     unless ( defined $method ) {
         my $protocolType = Lasso::Constants::MD_PROTOCOL_TYPE_SINGLE_LOGOUT;
         $method =
@@ -1366,7 +1489,6 @@ sub getDisplayType {
 # * HTTP parameter
 # * "samlIdPResolveCookie" cookie
 # * Rules
-# * Common Domain Cookie
 #
 # @return Array containing :
 # * IdP found (or undef)
@@ -1404,10 +1526,8 @@ sub getIDP {
         else {
             foreach ( keys %{ $self->idpList } ) {
                 my $idpConfKey = $self->idpList->{$_}->{confKey};
-                my $cond = $self->conf->{samlIDPMetaDataOptions}->{$idpConfKey}
-                  ->{samlIDPMetaDataOptionsResolutionRule};
-                next unless defined $cond;
-                if ( $self->safe->reval($cond) ) {
+                my $cond = $self->idpRules->{$idpConfKey} or next;
+                if ( $cond->( $req, $req->sessionInfo ) ) {
                     $self->logger->debug(
                         "IDP $idpConfKey resolution rule match");
                     $idp = $_;
@@ -1415,40 +1535,21 @@ sub getIDP {
                 }
             }
         }
-
-        # Case 5: use Common Domain Cookie
-        if (   !$idp
-            and $self->conf->{samlCommonDomainCookieActivation}
-            and $self->conf->{samlCommonDomainCookieReader} )
-        {
-            $self->logger->debug(
-                "Will try to use Common Domain Cookie for IDP resolution");
-
-            # Add current URL to CDC Reader URL
-            my $return_url = encode_base64( $self->self_url(), '' );
-
-            my $cdc_reader_url = $self->conf->{samlCommonDomainCookieReader};
-
-            $cdc_reader_url .= (
-                $self->conf->{samlCommonDomainCookieReader} =~ /\?/
-                ? '&u->confrl=' . $return_url
-                : '?url=' . $return_url
-            );
-
-            $self->logger->debug("Redirect user to $cdc_reader_url");
-
-            $req->urldc($cdc_reader_url);
-
-            $req->steps( [] );
-            return PE_REDIRECT;
-        }
-
         $self->logger->debug('No IDP found') unless ($idp);
     }
+    else {
+        $self->logger->debug("IDP $idp found from idp URL Parameter");
+    }
+
+    # Case 6: auto select IDP if only one IDP defined
+    if ( scalar keys %{ $self->idpList } == 1 ) {
+        ($idp) = keys %{ $self->idpList };
+        $self->logger->debug("Selecting the only defined SAML IDP: $idp");
+    }
 
     # Alert when selected IDP is unknown
     if ( $idp and !exists $self->idpList->{$idp} ) {
-        $self->userLogger->error("Required IDP $idp does not exists");
+        $self->logger->error("Required IDP $idp does not exists");
         $idp = undef;
     }
 
