@@ -458,14 +458,23 @@ sub validate {
     }
 
     # Get username
-    my $username = $localSession->data->{ $self->conf->{casAttr}
-          || $self->conf->{whatToTrace} };
+    my $app = $casServiceSession->data->{_casApp};
+    my $username_attribute =
+      (       $app
+          and $self->conf->{casAppMetaDataOptions}->{$app}
+          ->{casAppMetaDataOptionsUserAttribute} )
+      ? $self->conf->{casAppMetaDataOptions}->{$app}
+      ->{casAppMetaDataOptionsUserAttribute}
+      : (    $self->conf->{casAttr}
+          || $self->conf->{whatToTrace} );
+
+    my $username = $localSession->data->{$username_attribute};
 
     $self->logger->debug("Get username $username");
 
     # Return success message
     $self->deleteCasSession($casServiceSession);
-    return $self->returnCasValidateSuccess($username);
+    return $self->returnCasValidateSuccess( $req, $username );
 }
 
 sub proxy {
@@ -728,8 +737,16 @@ sub _validate2 {
     }
 
     # Get username
-    my $username = $localSession->data->{ $self->conf->{casAttr}
-          || $self->conf->{whatToTrace} };
+    my $username_attribute =
+      (       $app
+          and $self->conf->{casAppMetaDataOptions}->{$app}
+          ->{casAppMetaDataOptionsUserAttribute} )
+      ? $self->conf->{casAppMetaDataOptions}->{$app}
+      ->{casAppMetaDataOptionsUserAttribute}
+      : (    $self->conf->{casAttr}
+          || $self->conf->{whatToTrace} );
+
+    my $username = $localSession->data->{$username_attribute};
 
     $self->logger->debug("Get username $username");
 
