@@ -6,7 +6,7 @@ use Lemonldap::NG::Portal::Lib::Net::LDAP;
 
 extends 'Lemonldap::NG::Common::Module';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 # PROPERTIES
 
@@ -21,24 +21,23 @@ sub newLdap {
     my $ldap;
 
     # Build object and test LDAP connexion
-    if (
+    unless (
         $ldap = Lemonldap::NG::Portal::Lib::Net::LDAP->new(
             { p => $self->{p}, conf => $self->{conf} }
         )
-        and my $msg = $ldap->bind
       )
     {
-        if ( $msg->code != 0 ) {
-            $self->logger->error( "LDAP error: " . $msg->error );
-        }
-        else {
-            if ( $self->{conf}->{ldapPpolicyControl} and not $ldap->loadPP() ) {
-                $self->logger->error("LDAP password policy error");
-            }
-        }
+        $self->logger->error("LDAP initialization error: $@");
+        return undef;
     }
-    else {
-        $self->logger->error("LDAP error: $@");
+
+    # Test connection
+    my $msg = $ldap->bind;
+    if ( $msg->code ) {
+        $self->logger->error( 'LDAP test has failed: ' . $msg->error );
+    }
+    elsif ( $self->{conf}->{ldapPpolicyControl} and not $ldap->loadPP() ) {
+        $self->logger->error("LDAP password policy error");
     }
     return $ldap;
 }
@@ -49,30 +48,52 @@ has filter => (
     builder => 'buildFilter',
 );
 
-sub buildFilter {
-    my $conf = $_[0]->{conf};
-    $_[0]->{p}->logger->debug("LDAP Search base: $_[0]->{conf}->{ldapBase}");
+has mailFilter => (
+    is      => 'rw',
+    lazy    => 1,
+    builder => 'buildMailFilter',
+);
 
-    # TODO : mailLDAPFilter
-    my $filter =
-         $conf->{AuthLDAPFilter}
-      || $conf->{LDAPFilter}
-      || '(&(uid=$user)(objectClass=inetOrgPerson))';
+sub buildFilter {
+    return $_[0]->_buildFilter( $_[0]->conf->{AuthLDAPFilter}
+          || $_[0]->conf->{LDAPFilter}
+          || '(&(uid=$user)(objectClass=inetOrgPerson))' );
+}
+
+sub buildMailFilter {
+    my $f = $_[0]->conf->{mailLDAPFilter}
+      || '(&(mail=$user)(objectClass=inetOrgPerson))';
+    $f =~ s/\$mail\b/\$user/g;
+    return $_[0]->_buildFilter($f);
+}
+
+sub _buildFilter {
+    my ( $self, $filter ) = @_;
+    my $conf = $self->{conf};
+    $self->{p}->logger->debug("LDAP Search base: $_[0]->{conf}->{ldapBase}");
     $filter =~ s/"/\\"/g;
     $filter =~ s/\$(\w+)/".\$req->{sessionInfo}->{$1}."/g;
     $filter =~ s/\$req->\{sessionInfo\}->\{user\}/\$req->{user}/g;
     $filter =~
-      s/\$req->\{sessionInfo\}->\{(_?password|mail)\}/\$req->{datas}->{$1}/g;
-    $_[0]->{p}->logger->debug("LDAP transformed filter: $filter");
+      s/\$req->\{sessionInfo\}->\{(_?password|mail)\}/\$req->{data}->{$1}/g;
+    $self->{p}->logger->debug("LDAP transformed filter: $filter");
     $filter = "sub{my(\$req)=\$_[0];return \"$filter\";}";
-    return eval $filter;
+    my $res = eval $filter;
+
+    if ($@) {
+        $self->error("Unable to build fiter: $@");
+    }
+    return $res;
 }
 
 # INITIALIZATION
 
 sub init {
     my ($self) = @_;
-    $self->ldap and $self->filter;
+    $self->ldap
+      or $self->logger->error(
+        "LDAP initialization has failed, but let's continue");
+    $self->filter;
 }
 
 # RUNNING METHODS
@@ -83,6 +104,7 @@ sub bind {
     unless ($self->ldap
         and $self->ldap->root_dse( attrs => ['supportedLDAPVersion'] ) )
     {
+        $self->ldap->DESTROY if ( $self->ldap );
         $self->ldap( $self->newLdap );
     }
     return undef unless ( $self->ldap );
