@@ -19,7 +19,7 @@ use Mouse;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT);
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.4';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -1047,6 +1047,10 @@ sub getEndPointAuthenticationCredentials {
         $client_id     = $req->param('client_id');
         $client_secret = $req->param('client_secret');
     }
+    elsif ( $req->param('client_id') and !$req->param('client_secret') ) {
+        $self->logger->debug("Method none used");
+        $client_id = $req->param('client_id');
+    }
 
     return ( $client_id, $client_secret );
 }
@@ -1161,6 +1165,11 @@ sub createJWT {
         my $client_secret =
           $self->conf->{oidcRPMetaDataOptions}->{$rp}
           ->{oidcRPMetaDataOptionsClientSecret};
+        unless ($client_secret) {
+            $self->logger->error(
+                "Algorithm $alg needs a Client Secret to sign JWT");
+            return;
+        }
 
         my $digest;
 
@@ -1190,6 +1199,12 @@ sub createJWT {
 
         # Get signing private key
         my $priv_key = $self->conf->{oidcServicePrivateKeySig};
+        unless ($priv_key) {
+            $self->logger->error(
+                "Algorithm $alg needs a Private Key to sign JWT");
+            return;
+        }
+
         my $rsa_priv = Crypt::OpenSSL::RSA->new_private_key($priv_key);
 
         if ( $alg eq "RS256" ) {
@@ -1385,6 +1400,52 @@ sub addRouteFromConf {
     }
 }
 
+# Validate PKCE code challenge with given code challenge method
+# @param code_verifier
+# @param code_challenge
+# @param code_challenge_method
+# @return boolean 1 if challenge succeed, 0 else
+sub validatePKCEChallenge {
+    my ( $self, $code_verifier, $code_challenge, $code_challenge_method ) = @_;
+
+    unless ($code_verifier) {
+        $self->logger->error("PKCE required but no code verifier provided");
+        return 0;
+    }
+
+    $self->logger->debug("PKCE code verifier received: $code_verifier");
+
+    if ( !$code_challenge_method or $code_challenge_method eq "plain" ) {
+        if ( $code_verifier eq $code_challenge ) {
+            $self->logger->debug("PKCE challenge validated (plain method)");
+            return 1;
+        }
+        else {
+            $self->logger->error("PKCE challenge failed (plain method)");
+            return 0;
+        }
+    }
+
+    elsif ( $code_challenge_method eq "S256" ) {
+        my $code_verifier_hashed = encode_base64url( sha256($code_verifier) );
+        if ( $code_verifier_hashed eq $code_challenge ) {
+            $self->logger->debug("PKCE challenge validated (S256 method)");
+            return 1;
+        }
+        else {
+            $self->logger->error("PKCE challenge failed (S256 method)");
+            return 0;
+        }
+    }
+
+    else {
+        $self->logger->error("PKCE challenge method not valid");
+        return 0;
+    }
+
+    return 0;
+}
+
 1;
 
 __END__
@@ -1543,6 +1604,10 @@ Build Logout Response URI
 Build a Lemonldap::NG::Common::PSGI::Router route from OIDC configuration
 attribute
 
+=head2 validatePKCEChallenge
+
+Validate PKCE code challenge with given code challenge method
+
 =head1 SEE ALSO
 
 L<Lemonldap::NG::Portal::AuthOpenIDConnect>, L<Lemonldap::NG::Portal::UserDBOpenIDConnect>
