@@ -8,7 +8,7 @@
 #                  of lemonldap-ng.ini) and underlying handler configuration
 package Lemonldap::NG::Portal::Main::Init;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.3';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -22,11 +22,13 @@ use Regexp::Assemble;
 has localConfig => ( is => 'rw', default => sub { {} } );
 has conf        => ( is => 'rw', default => sub { {} } );
 has menu        => ( is => 'rw', default => sub { {} } );
+has trOver      => ( is => 'rw', default => sub { { all => {} } } );
 
 # Sub modules
 has _authentication => ( is => 'rw' );
 has _userDB         => ( is => 'rw' );
 has _passwordDB     => ( is => 'rw' );
+has _sfEngine       => ( is => 'rw' );
 
 has loadedModules => ( is => 'rw' );
 
@@ -39,30 +41,40 @@ has _jsRedirect => ( is => 'rw' );
 has trustedDomainsRe => ( is => 'rw' );
 
 # Lists to store plugins entry-points
-has beforeAuth => (
+my @entryPoints;
+
+BEGIN {
+    @entryPoints = (
+
+        # Auth process entrypoints
+        qw(beforeAuth betweenAuthAndData afterData endAuth),
+
+        # Authenticated users entrypoint
+        'forAuthUser',
+
+        # Logout entrypoint
+        'beforeLogout',
+
+        # Special endpoint
+        'authCancel',    # Clean pdata when user click on "cancel"
+    );
+
+    foreach (@entryPoints) {
+        has $_ => (
+            is      => 'rw',
+            isa     => 'ArrayRef',
+            default => sub { [] }
+        );
+    }
+}
+
+# Endpoints inserted after any main sub
+has 'afterSub'  => ( is => 'rw', default => sub { {} } );
+has 'aroundSub' => ( is => 'rw', default => sub { {} } );
+
+has spRules => (
     is      => 'rw',
-    isa     => 'ArrayRef',
-    default => sub { [] }
-);
-has betweenAuthAndDatas => (
-    is      => 'rw',
-    isa     => 'ArrayRef',
-    default => sub { [] }
-);
-has afterDatas => (
-    is      => 'rw',
-    isa     => 'ArrayRef',
-    default => sub { [] }
-);
-has forAuthUser => (
-    is      => 'rw',
-    isa     => 'ArrayRef',
-    default => sub { [] }
-);
-has beforeLogout => (
-    is      => 'rw',
-    isa     => 'ArrayRef',
-    default => sub { [] }
+    default => sub { {} }
 );
 
 # Custom template parameters
@@ -76,30 +88,47 @@ has csp => ( is => 'rw' );
 sub init {
     my ( $self, $args ) = @_;
     $args ||= {};
-    $self->localConfig(
-        {
+    $self->localConfig( {
             %{ Lemonldap::NG::Common::Conf->new( $args->{configStorage} )
                   ->getLocalConf('portal')
             },
             %$args
         }
     );
+
+    # Load override messages from lemonldap-ng.ini
     foreach my $k ( keys %{ $self->localConfig } ) {
         if ( $k =~ /tpl_(.*)/ ) {
             $self->customParameters->{$1} = $self->localConfig->{$k};
         }
+        elsif ( $k =~ /error_(?:(\w+?)_)?(\d+)$/ ) {
+            my $lang = $1 || 'all';
+            $self->trOver->{$lang}->{"PE$2"} = $self->localConfig->{$k};
+        }
+        elsif ( $k =~ /msg_(?:(\w+?)_)?(\w+)$/ ) {
+            my $lang = $1 || 'all';
+            $self->trOver->{$lang}->{$2} = $self->localConfig->{$k};
+        }
     }
+    $self->trOver( JSON::to_json( $self->trOver ) );
 
     # Purge loaded module list
     $self->loadedModules( {} );
+
+    # Insert `reloadConf` in handler reload stack
     Lemonldap::NG::Handler::Main->onReload( $self, 'reloadConf' );
+
+    # Handler::PSGI::Try initialization
     return 0 unless ( $self->SUPER::init( $self->localConfig ) );
-    return 0 if ( $self->error );
+    if ( $self->error ) {
+        $self->logger->error( $self->error );
+        return 0;
+    }
 
     # Handle requests (other path may be declared in enabled plugins)
     $self
 
-      # "/"
+      # "/" or undeclared paths
       ->addUnauthRoute( '*' => 'login',     ['GET'] )
       ->addUnauthRoute( '*' => 'postLogin', ['POST'] )
       ->addAuthRoute( '*' => 'authenticatedRequest',     ['GET'] )
@@ -109,6 +138,14 @@ sub init {
       ->addUnauthRoute( 'psgi.js' => 'sendJs', ['GET'] )
       ->addAuthRoute( 'psgi.js' => 'sendJs', ['GET'] )
 
+      # portal.css
+      ->addUnauthRoute( 'portal.css' => 'sendCss', ['GET'] )
+      ->addAuthRoute( 'portal.css' => 'sendCss', ['GET'] )
+
+      # lmerror
+      ->addUnauthRoute( lmerror => { ':code' => 'lmError' }, ['GET'] )
+      ->addAuthRoute( lmerror => { ':code' => 'lmError' }, ['GET'] )
+
       # Core REST API
       ->addUnauthRoute( ping => 'pleaseAuth', ['GET'] )
       ->addAuthRoute( ping => 'authenticated', ['GET'] )
@@ -122,7 +159,6 @@ sub init {
     # Default routes must point to routines declared above
     $self->defaultAuthRoute('');
     $self->defaultUnauthRoute('');
-
     return 1;
 }
 
@@ -133,12 +169,10 @@ sub reloadConf {
     %{ $self->{conf} } = %{ $self->localConfig };
 
     # Reinitialize arrays
-    foreach (
-        qw(_macros _groups beforeAuth betweenAuthAndDatas afterDatas forAuthUser beforeLogout)
-      )
-    {
+    foreach ( qw(_macros _groups), @entryPoints ) {
         $self->{$_} = [];
     }
+    $self->spRules( {} );
 
     # Load conf in portal object
     foreach my $key ( keys %$conf ) {
@@ -147,7 +181,7 @@ sub reloadConf {
 
     # Initialize content-security-policy header
     my $csp = '';
-    foreach (qw(default img src style font connect)) {
+    foreach (qw(default img src style font connect script)) {
         my $prm = $self->conf->{ 'csp' . ucfirst($_) };
         $csp .= "$_-src $prm;" if ($prm);
     }
@@ -156,6 +190,12 @@ sub reloadConf {
     # Initialize templateDir
     $self->{templateDir} =
       $self->conf->{templateDir} . '/' . $self->conf->{portalSkin};
+    unless ( -d $self->{templateDir} ) {
+        $self->error("Template dir $self->{templateDir} doesn't exist");
+        return $self->fail;
+    }
+    $self->templateDir(
+        [ $self->{templateDir}, $self->conf->{templateDir} . '/bootstrap' ] );
 
     $self->{staticPrefix} = $self->conf->{staticPrefix} || '/static';
     $self->{languages}    = $self->conf->{languages}    || '/';
@@ -182,6 +222,11 @@ sub reloadConf {
     }
     $self->conf->{domain} =~ s/^([^\.])/.$1/;
 
+    # Load menu
+    # ---------
+    $self->menu( $self->loadPlugin('::Main::Menu') );
+    $self->displayInit;
+
     # Load authentication/userDB
     # --------------------------
     my $mod;
@@ -190,7 +235,8 @@ sub reloadConf {
             $self->error("$type is not set");
             return $self->fail;
         }
-        $mod = $self->conf->{$type} unless ( $self->conf->{$type} eq 'Same' );
+        $mod = $self->conf->{$type}
+          unless ( $self->conf->{$type} eq 'Same' );
         my $module = '::' . ucfirst($type) . '::' . $mod;
         $module =~ s/Authentication/Auth/;
 
@@ -199,6 +245,11 @@ sub reloadConf {
           unless ( $self->{"_$type"} = $self->loadPlugin($module) );
     }
 
+    # Load second-factor engine
+    return $self->fail
+      unless $self->{_sfEngine} =
+      $self->loadPlugin( $self->conf->{'sfEngine'} );
+
     # Initialize trusted domain regexp
     if (    $self->conf->{trustedDomains}
         and $self->conf->{trustedDomains} =~ /^\s*\*\s*$/ )
@@ -225,6 +276,9 @@ sub reloadConf {
                 $re->add("$_");
             }
         }
+        my $p = $self->conf->{portal};
+        $p =~ s#https?://([^/]*).*$#$1#;
+        $re->add( quotemeta($p) );
         foreach my $vhost ( keys %{ $self->conf->{locationRules} } ) {
             $self->logger->debug("Vhost $vhost added in trusted domains");
             $re->add( quotemeta($vhost) );
@@ -266,45 +320,124 @@ sub reloadConf {
       or $self->logger->error(
         'jsRedirect returns an error: ' . HANDLER->tsv->{jail}->error );
 
-    $self->menu( $self->loadPlugin('::Main::Menu') );
-    $self->displayInit;
-
     # Load plugins
     foreach my $plugin ( $self->enabledPlugins ) {
         $self->loadPlugin($plugin) or return $self->fail;
     }
 
+    # Clean $req->pdata after authentication
+    push @{ $self->endAuth }, sub {
+        unless ( $_[0]->pdata->{keepPdata} ) {
+            $self->logger->debug('Cleaning pdata');
+            $_[0]->pdata( {} );
+        }
+        return PE_OK;
+    };
+    unshift @{ $self->beforeAuth }, sub {
+        if ( $_[0]->param('cancel') ) {
+            $self->logger->debug('Cancel called, push authCancel calls');
+            unshift @{ $_[0]->steps }, @{ $self->authCancel };
+            return PE_OK;
+        }
+    };
+    my $portal = $self->conf->{portal};
+    $portal =~ s#^https?://(.*?)(?:[:/].*)?$#$1#;
+    HANDLER->tsv->{defaultCondition}->{$portal} ||= sub { 1 };
+
     1;
 }
 
+# Method used to load plugins
 sub loadPlugin {
     my ( $self, $plugin ) = @_;
+    unless ($plugin) {
+        require Carp;
+        Carp::confess('Calling loadPugin without arg !');
+    }
     my $obj;
     return 0
       unless ( $obj = $self->loadModule("$plugin") );
     return $self->findEP( $plugin, $obj );
 }
 
+# Insert declared entry points into corresponding arrays
 sub findEP {
     my ( $self, $plugin, $obj ) = @_;
-    foreach my $sub (
-        qw(beforeAuth betweenAuthAndDatas afterDatas forAuthUser beforeLogout))
-    {
+
+    # Standards entry points
+    foreach my $sub (@entryPoints) {
         if ( $obj->can($sub) ) {
             $self->logger->debug(" Found $sub entry point:");
             if ( my $callback = $obj->$sub ) {
-                push @{ $self->{$sub} }, sub { $obj->$callback( $_[0] ) };
+                push @{ $self->{$sub} }, sub {
+                    eval {
+                        $obj->logger->debug("Launching ${plugin}::$callback");
+                    };
+                    $obj->$callback(@_);
+                };
                 $self->logger->debug("  -> $callback");
             }
         }
     }
-    ( $obj and $obj->init ) or return 0;
+    if ( $obj->can('afterSub') ) {
+        $self->logger->debug("Found afterSub in $plugin");
+        my $h = $obj->afterSub;
+        unless ( ref $h and ref($h) eq 'HASH' ) {
+            $self->logger->error(
+                '"afterSub" endpoint must be a hashref, skipped');
+        }
+        else {
+            foreach my $ep ( keys %$h ) {
+                my $callback = $h->{$ep};
+                push @{ $self->afterSub->{$ep} }, sub {
+                    eval {
+                        $obj->logger->debug(
+                            "Launching ${plugin}::$callback afterSub $ep");
+                    };
+                    $obj->$callback(@_);
+                };
+            }
+        }
+    }
+    if ( $obj->can('aroundSub') ) {
+        $self->logger->debug("Found aroundSub in $plugin");
+        my $h = $obj->aroundSub;
+        unless ( ref $h and ref($h) eq 'HASH' ) {
+            $self->logger->error(
+                '"aroundSub" endpoint must be a hashref, skipped');
+        }
+        else {
+            foreach my $ep ( keys %$h ) {
+                my $callback = $h->{$ep};
+                my $previousSub = $self->aroundSub->{$ep} ||= sub {
+                    $self->logger->debug(
+                        "$ep launched inside ${plugin}::$callback");
+                    $self->$ep(@_);
+                };
+                $self->aroundSub->{$ep} = sub {
+                    $self->logger->debug(
+                        "Launching ${plugin}::$callback instead of $ep");
+                    $obj->$callback( $previousSub, @_ );
+                };
+            }
+        }
+    }
     $self->logger->debug("Plugin $plugin initializated");
+
+    # Rules for menu
+    if ( $obj->can('spRules') ) {
+        foreach my $k ( keys %{ $obj->{spRules} } ) {
+            $self->logger->info(
+"$k is defined more than one time, it can have some bad effect on Menu display"
+            ) if ( $self->spRules->{$k} );
+            $self->spRules->{$k} = $obj->{spRules}->{$k};
+        }
+    }
     return $obj;
 }
 
 sub loadModule {
-    my ( $self, $module, $conf ) = @_;
+    my ( $self, $module, $conf, %args ) = @_;
     $conf //= $self->conf;
     my $obj;
     $module = "Lemonldap::NG::Portal$module" if ( $module =~ /^::/ );
@@ -315,13 +448,14 @@ sub loadModule {
         return 0;
     }
     eval {
-        $obj = $module->new( { p => $self, conf => $conf } );
+        $obj = $module->new( { p => $self, conf => $conf, %args } );
         $self->logger->debug("Module $module loaded");
     };
     if ($@) {
         $self->error("Unable to build $module object: $@");
         return 0;
     }
+    ( $obj and $obj->init ) or return 0;
     $self->loadedModules->{$module} = $obj;
     return $obj;
 }
@@ -335,8 +469,8 @@ sub fail {
 
 sub displayError {
     my ( $self, $req ) = @_;
-    return $self->sendError( $req, 'Portal error, contact your administrator',
-        500 );
+    return $self->sendError( $req,
+        'Portal error, contact your administrator', 500 );
 }
 
 1;
