@@ -1,6 +1,6 @@
 package Lemonldap::NG::Portal::Main::Process;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.3';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -15,18 +15,27 @@ use POSIX qw(strftime);
 # to a subroutine (see Lemonldap::NG::Portal::Main::Run.pm)
 
 sub process {
-    my ( $self, $req ) = @_;
+    my ( $self, $req, %args ) = @_;
 
-    #$req->error(PE_OK);
+    # Store ipAddr in env
+    $req->env->{ipAddr} = $req->address;
     my $err = PE_OK;
     while ( my $sub = shift @{ $req->steps } ) {
         if ( ref $sub ) {
             $self->logger->debug("Processing code ref");
-            last if ( $err = $sub->($req) );
+            last if ( $err = $sub->( $req, %args ) );
         }
         else {
             $self->logger->debug("Processing $sub");
-            last if ( $err = $self->$sub($req) );
+            if ( my $as = $self->aroundSub->{$sub} ) {
+                last if ( $err = $as->( $req, %args ) );
+            }
+            else {
+                last if ( $err = $self->$sub( $req, %args ) );
+            }
+            if ( $self->afterSub->{$sub} ) {
+                unshift @{ $req->steps }, @{ $self->afterSub->{$sub} };
+            }
         }
     }
     $self->logger->debug("Returned error: $err") if ($err);
@@ -36,14 +45,14 @@ sub process {
 # First process block: check args
 # -------------------------------
 
-# For post requests, parse datas
+# For post requests, parse data
 sub restoreArgs {
     my ( $self, $req ) = @_;
     $req->mustRedirect(1);
     return PE_OK;
 }
 
-sub importHandlerDatas {
+sub importHandlerData {
     my ( $self, $req ) = @_;
     $req->{sessionInfo} = $req->userData;
     $req->id( $req->sessionInfo->{_session_id} );
@@ -73,8 +82,9 @@ sub controlUrl {
             }
         }
     }
-    $req->{datas}->{_url} ||= '';
-    if ( my $url = $req->param('url') ) {
+    $req->{data}->{_url} ||= '';
+    my $url = $req->param('url') || $req->pdata->{_url};
+    if ($url) {
 
         # REJECT NON BASE64 URL
         if ( $req->urlNotBase64 ) {
@@ -90,7 +100,7 @@ sub controlUrl {
             $req->{urldc} =~ s/[\r\n]//sg;
         }
 
-        # For logout request, test if Referer comes from an authorizated site
+        # For logout request, test if Referer comes from an authorized site
         my $tmp = (
               $req->param('logout')
             ? $req->referer
@@ -109,7 +119,7 @@ sub controlUrl {
             return PE_BADURL;
         }
 
-        # Non protected hosts
+        # Unprotected hosts
         if ( $tmp and !$self->isTrustedUrl($tmp) ) {
             $self->userLogger->error(
                     "URL contains a non protected host (param: "
@@ -119,7 +129,10 @@ sub controlUrl {
             return PE_BADURL;
         }
 
-        $req->datas->{_url} = $url;
+        $req->env->{urldc}  = $req->{urldc};
+        $req->env->{_url}   = $req->{_url};
+        $req->data->{_url}  = $url;
+        $req->pdata->{_url} = $url;
     }
 
     PE_OK;
@@ -136,40 +149,48 @@ sub checkLogout {
 
 sub authLogout {
     my ( $self, $req ) = @_;
-    return $self->_authentication->authLogout($req);
+    my $res = $self->_authentication->authLogout($req);
+    unless ( $res or $req->pdata->{keepPdata} ) {
+        $self->logger->debug('Cleaning pdata');
+        $req->pdata( {} );
+    }
+    return $res;
 }
 
 sub deleteSession {
     my ( $self, $req ) = @_;
-    $req->userData( {} );
-    my $apacheSession = $self->getApacheSession( $req->id );
-    my $id            = $req->id;
-    unless ($apacheSession) {
-        $self->logger->debug("Session $id already deleted");
-        return PE_OK;
-    }
-    unless ( $self->_deleteSession( $req, $apacheSession ) ) {
-        $self->logger->error("Unable to delete session $id");
-        $self->logger->error( $apacheSession->error );
-        return PE_ERROR;
-    }
-    else {
-        $self->logger->debug("Session $id deleted from global storage");
+    if ( my $id = $req->id || $req->userData->{_session_id} ) {
+        my $apacheSession = $self->getApacheSession( $req->id );
+        unless ($apacheSession) {
+            $self->logger->debug("Session $id already deleted");
+            return PE_OK;
+        }
+        unless ( $self->_deleteSession( $req, $apacheSession ) ) {
+            $self->logger->error("Unable to delete session $id");
+            $self->logger->error( $apacheSession->error );
+            return PE_ERROR;
+        }
+        else {
+            $self->logger->debug("Session $id deleted from global storage");
+        }
     }
 
     # TODO
     # Collect logout services and build hidden iFrames
-    if ( $req->datas->{logoutServices} and %{ $req->datas->{logoutServices} } )
-    {
+    if ( $req->data->{logoutServices} and %{ $req->data->{logoutServices} } ) {
 
         $self->logger->debug("Create iFrames to forward logout to services");
 
-        $req->info('<h3 trmsg="logoutFromOtherApp"></h3>');
+        $req->info(
+            $self->loadTemplate(
+                'simpleInfo', params => { trspan => 'logoutFromOtherApp' }
+            )
+        );
 
-        foreach ( keys %{ $req->datas->{logoutServices} } ) {
+        foreach ( keys %{ $req->data->{logoutServices} } ) {
             my $logoutServiceName = $_;
             my $logoutServiceUrl =
-              $req->datas->{logoutServices}->{$logoutServiceName};
+              $req->data->{logoutServices}->{$logoutServiceName};
 
             $self->logger->debug(
                 "Find logout service $logoutServiceName ($logoutServiceUrl)");
@@ -189,6 +210,7 @@ sub deleteSession {
             $req->urldc( $req->script_name . "?logout=1" );
         }
     }
+    $req->userData( {} );
 
     # Redirect or Post if asked by authLogout
     if ( $req->urldc and $req->urldc ne $self->conf->{portal} ) {
@@ -238,6 +260,7 @@ sub checkXSSAttack {
 
 sub extractFormInfo {
     my ( $self, $req ) = @_;
+    return PE_ERROR unless ( $self->_authentication );
     my $ret = $self->_authentication->extractFormInfo($req);
     if ( $ret == PE_OK and not( $req->user or $req->continue ) ) {
         $self->logger->error(
@@ -253,22 +276,44 @@ sub extractFormInfo {
                 value   => 0,
                 domain  => $self->conf->{domain},
                 secure  => 0,
-                expires => '-1d',
+                expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
             )
         );
-        return PE_SESSIONEXPIRED;
+        if ( $self->conf->{portalErrorOnExpiredSession} ) {
+            $ret = PE_SESSIONEXPIRED;
+        }
     }
     return $ret;
 }
 
 sub getUser {
-    my ( $self, $req ) = @_;
-    return $self->_userDB->getUser($req);
+    my ( $self, $req, %args ) = @_;
+    return PE_ERROR unless ( $self->_userDB );
+    return $self->_userDB->getUser( $req, %args );
 }
 
 sub authenticate {
     my ( $self, $req ) = @_;
-    return $req->authResult( $self->_authentication->authenticate($req) );
+    my $ret = $req->authResult( $self->_authentication->authenticate($req) );
+    $self->logger->debug( " -> authResult = " . $req->authResult );
+    if ( $ret == PE_OK ) {
+        $req->{sessionInfo}->{_lastAuthnUTime} = time();
+        return $ret;
+    }
+
+    # Store failed login into history
+    $req->steps( [
+            'setSessionInfo',           'setMacros',
+            'setPersistentSessionInfo', 'storeHistory',
+            @{ $self->afterData }, sub { PE_BADCREDENTIALS }
+        ]
+    );
+
+    # Ignore result, process will end at least with PE_BADCREDENTIALS
+    my $tmp = $self->process($req);
+    $ret = $tmp if ( $tmp == PE_WAIT );
+
+    return $ret;
 }
 
 # Third block: Session data providing
@@ -300,26 +345,17 @@ sub setSessionInfo {
 
     # Date and time
     if ( $self->conf->{updateSession} ) {
-        $req->{sessionInfo}->{updateTime} =
+        $req->{sessionInfo}->{_updateTime} =
           strftime( "%Y%m%d%H%M%S", localtime() );
     }
     else {
         $req->{sessionInfo}->{_utime} ||= time();
-        $req->{sessionInfo}->{startTime} =
+        $req->{sessionInfo}->{_startTime} =
           strftime( "%Y%m%d%H%M%S", localtime() );
         $req->{sessionInfo}->{_lastSeen} = time()
           if $self->conf->{timeoutActivity};
     }
 
-    # Get environment variables matching exportedVars (works only with HTTP_*
-    # and SSL_*: see Main/Request.pm)
-    foreach ( keys %{ $self->conf->{exportedVars} } ) {
-        if ( my $tmp = $req->{ $self->conf->{exportedVars}->{$_} } ) {
-            $tmp =~ s/[\r\n]/ /gs;
-            $req->{sessionInfo}->{$_} = $tmp;
-        }
-    }
-
     # Store URL origin in session
     $req->{sessionInfo}->{_url} = $req->{urldc};
 
@@ -336,7 +372,7 @@ sub setMacros {
     my ( $self, $req ) = @_;
     foreach ( sort keys %{ $self->_macros } ) {
         $req->{sessionInfo}->{$_} =
-          $self->_macros->{$_}->( $req->sessionInfo );
+          $self->_macros->{$_}->( $req, $req->sessionInfo );
     }
     PE_OK;
 }
@@ -375,7 +411,7 @@ sub setPersistentSessionInfo {
 sub setLocalGroups {
     my ( $self, $req ) = @_;
     foreach ( sort keys %{ $self->_groups } ) {
-        if ( $self->_groups->{$_}->( $req->sessionInfo ) ) {
+        if ( $self->_groups->{$_}->( $req, $req->sessionInfo ) ) {
             $req->{sessionInfo}->{groups} .=
               $self->conf->{multiValuesSeparator} . $_;
             $req->{sessionInfo}->{hGroups}->{$_}->{name} = $_;
@@ -385,7 +421,7 @@ sub setLocalGroups {
     # Clear values separator at the beginning
     if ( $req->{sessionInfo}->{groups} ) {
         $req->{sessionInfo}->{groups} =~
-          s/^$self->conf->{multiValuesSeparator}//o;
+          s/^$self->{conf}->{multiValuesSeparator}//o;
     }
     PE_OK;
 }
@@ -465,4 +501,17 @@ sub buildCookie {
     PE_OK;
 }
 
+sub secondFactor {
+    my ( $self, $req ) = @_;
+    return $self->_sfEngine->run($req);
+}
+
+sub storeHistory {
+    my ( $self, $req ) = @_;
+    if ( $self->conf->{loginHistoryEnabled} ) {
+        $self->registerLogin($req);
+    }
+    PE_OK;
+}
+
 1;
